%% flat model training
%% Azad Naik

clc
clear all
close all

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'dmoz_2010_small';

%% choose split number
chooseSplit = 0;

%% User parameter Regularization C, Bias B
C = [0.001 0.01 0.1 1 10 100 1000];
B = 1;

%% 7 for L2 regularized logistic regression (dual), 0 (for primal)
modelLearn = 0;

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');
addpath(fullPath);

%% Select path based on Operating System
if ispc
    partialPath = '../../result/';
elseif isunix
	partialPath = '/home/anaik3/code/flat/result/';
else
    % other operating system
end 

%% Choose output folder name
resultPath = strcat(partialPath, datasetFolderName);
fn = fullfile(resultPath);
if exist(fn, 'dir')
   warning('directory exist- do you want to overwrite?');
   prompt = 'enter 1 for yes; any other key for no \n';
   userChoice = input(prompt);
   if (userChoice ~= 1)
       display('please choose different directory or overwrite existing directory...exiting program');
       return;
   end
   display('overwriting existing folder');
else
   mkdir(fn);
end

%% load training file in liblinear format 
filename = ['train' num2str(chooseSplit) '.txt'];
filePathTrain = strcat(fullPath, '/', filename);
[label_vector, instance_matrix] = libsvmread(filePathTrain);
[id d] = size(instance_matrix);
fprintf('Finished Reading train File\n');

%% load validation file in liblinear format 
filenameT = ['validation' num2str(chooseSplit) '.txt'];
filePathTest = strcat(fullPath, '/', filenameT);
[label_vectorT, instance_matrixT] = libsvmread(filePathTest);
fprintf('finished reading validation file\n');

%% Total number of possible labels
labelData = unique(sort(label_vector));

%% Timer variable
stopTime = 0;

%% best model with best macroF1
bestModel = 0;
bestMacroF1 = 0;
    
for regTest = 1:length(C)
    cmd = ['-c ', num2str(C(regTest)), ' -s ', num2str(modelLearn), ' -B ', num2str(B)];

    %% train Model
    startTime = tic;
    model = ovrtrain(label_vector, instance_matrix, cmd);
    stopTime = stopTime + toc(startTime);      

    %% validate Model
    startTime = tic;
    [pred ac decv] = ovrpredict(label_vectorT, instance_matrixT, model);
    stopTime = stopTime + toc(startTime);

    %% check if this is the best model
    [microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(label_vectorT, pred, labelData(:, 1));             
    if(macroF1Val >= bestMacroF1)
        bestMacroF1 = macroF1Val;
        bestModel = model;
    end
end

filename = [num2str(chooseSplit) 'flat_' datasetFolderName '.mat'];
fullFileName = fullfile(fn, filename);
save(fullFileName, '-struct', 'bestModel');

sprintf('time taken for model learning: %0.5f mins', stopTime/60)


% C = confusionmat(dataDMOZtest(:, 1), pred);                   %# confusion matrix
