%% one vs all; flat/hierarchical measures computation
%% Azad Naik

clc
clear all
close all

%% Result path
resultPath = '../../result/';

%% Test dataset path
datasetPath = '../../data/';

%% Dataset folder name
datasetFolderName = 'dmoz_2010_small';

%% choose split number
chooseSplit = 1;

if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;    
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;
else
end

%% bias
B = 1;

%% 7 for L2 regularized logistic regression (dual), 0 (for primal)
modelLearn = 0;

%% add test dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% load Test file
filenameT = 'test.txt';
filePathTest = strcat(fullPath, '/', filenameT);
[label_vectorT, instance_matrixT] = libsvmread(filePathTest);
[idT dT] = size(instance_matrixT);
fprintf('Finished Reading test File\n');

%% add result path
fullPath = strcat(resultPath, datasetFolderName);
addpath(fullPath);

% Each level prediction is stored in predictedIndex variable last columns is true index
predictedIndex = zeros(idT, 1);

%% Load label file (mapping.txt file contains all leaf node labels)
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    labelSet = unique(label_vectorTrain);
else
    filename = 'mapping.txt';
    labelSet = load(filename);
    labelSet = sort(labelSet);
end

%% perform prediction
filename = [num2str(chooseSplit) 'flat_' datasetFolderName '.mat'];
model = load(filename);
startTime = tic;
[pred ac decv] = ovrpredict(label_vectorT, instance_matrixT, model);
stopTime = toc(startTime);

%% calculate various measures
[microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(label_vectorT, pred, labelSet(:, 1));

microF1Val
macroF1Val

%% Load Hierarchy File [EdgeNode1, EdgeNode2]
dataHierarchy = load('cat_hier.txt');
[Nh dh] = size(dataHierarchy);

leafNodes = size(Nh, 2);
leafNodes = dataHierarchy(:, 2);

leafNodes = sort(leafNodes);
leafNodes = unique(leafNodes);
[Nleaf dLeaf] = size(leafNodes);

for i = 1:Nleaf
    a = find(dataHierarchy(:, 1) == leafNodes(i, 1));
    if(size(a, 1) > 0)
        %% Internal Node 
        leafNodes(i, 2) = 0;
    else
        %% Leaf Node
        leafNodes(i, 2) = 1;
    end
end

[r, c, v] = find(leafNodes(:, 2) == 1);
leafNodes = leafNodes(r, 1);
[Nleaf dLeaf] = size(leafNodes);


%% contains leaf node to root info
chainMatrix = zeros(Nleaf, noOfLevels+1);
chainMatrix(:, 1) = leafNodes(:, 1);

for i = 1:Nleaf
    dataHierarchyLength = 1;
    toFind = leafNodes(i, 1);
    for j = 1:noOfLevels+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

leafNodes = size(Nh, 2);
leafNodes = dataHierarchy(:, 2);

leafNodes = sort(leafNodes);
leafNodes = unique(leafNodes);
[Nleaf dLeaf] = size(leafNodes);

for i = 1:Nleaf
    a = find(dataHierarchy(:, 1) == leafNodes(i, 1));
    if(size(a, 1) > 0)
        %% Internal Node 
        leafNodes(i, 2) = 0;
    else
        %% Leaf Node
        leafNodes(i, 2) = 1;
    end
end

[r, c, v] = find(leafNodes(:, 2) == 1);
leafNodes = leafNodes(r, 1);
[Nleaf dLeaf] = size(leafNodes);


%% contains leaf node to root info
chainMatrix = zeros(Nleaf, noOfLevels+1);
chainMatrix(:, 1) = leafNodes(:, 1);

for i = 1:Nleaf
    dataHierarchyLength = 1;
    toFind = leafNodes(i, 1);
    for j = 1:noOfLevels+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

%% Load Label Data File [Actual Label, Predicted Label]
dataLabel = [label_vectorT pred]; 
[Nl dl] = size(dataLabel);

%% predict Hierarchical Precision/Recall
hPDenominator = 0; hPRNumerator = 0;
hRDenominator = 0; 
for i = 1:Nl
    %% True Label
    a = find(chainMatrix(:, 1) == dataLabel(i, 1));
    %% Predicted Label
    b = find(chainMatrix(:, 1) == dataLabel(i, 2));
    sizehP = size(find(chainMatrix(b, :) > 0), 2);
    sizehR = size(find(chainMatrix(a, :) > 0), 2);
    hPDenominator = hPDenominator + sizehP;
    hRDenominator = hRDenominator + sizehR;
    for j = 1:sizehP
        hPRNumerator = hPRNumerator + size(find(ismember(chainMatrix(a, :), chainMatrix(b, j)) > 0), 2);
    end  
    hPRNumerator = hPRNumerator;
end
 
hierarchicalPrecision = hPRNumerator/hPDenominator
hierarchicalRecall = hPRNumerator/hRDenominator
hierarchicalF1 = (2*hierarchicalPrecision*hierarchicalRecall)/(hierarchicalPrecision + hierarchicalRecall)

%% Tree-induced Error
TE = 0;
for i = 1:Nl
    if(dataLabel(i, 1) == dataLabel(i, 2))
        %% No Error
        TE = TE + 0;
    else
        %% True Label
        a = find(chainMatrix(:, 1) == dataLabel(i, 1));
        sizea = size(find(chainMatrix(a, :) > 0), 2);
        %% Predicted Label
        b = find(chainMatrix(:, 1) == dataLabel(i, 2));
        sizeb = size(find(chainMatrix(b, :) > 0), 2);
        foundFlag = 0;
        for j = 1:sizea
           found = size(find(ismember(chainMatrix(a, j), chainMatrix(b, :)) > 0), 2); 
           if(found > 0)
               %%found
               TE = TE + (j-1) + (find(chainMatrix(b, :) == chainMatrix(a, j)) - 1);
               foundFlag = 1;
               break;
           else
           end    
        end
        if(foundFlag == 0)
            TE = TE + sizea + sizeb;
        end
    end
end

TE = TE/Nl

sprintf('time taken for test dataset prediction: %0.5f mins', stopTime/60)

%% save predicted label
filename = [num2str(chooseSplit) 'test_prediction_' datasetFolderName '_flat.txt'];
fullFileName = fullfile(fullPath, filename);
dlmwrite(fullFileName, pred, 'precision', '%d');  
