%% f-score computation
%% Azad Naik

clc
clear all
close all

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'ng';

%% choose split number
chooseSplit = 0;

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
    C = [1 0.1 10];  %% best parameter
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
    C = [1000 1000 10];  %% best parameter
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
    C = [10 10 10];  %% best parameter
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
    C = [1000 1000 1000 1000 1000]; %% best parameter
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;
    C = [1000 1000 1000 1000 1000]; %% best parameter
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    noOfLevels = 5;
    C = [1000 1000 1000 1000 1000]; %% best parameter
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;
    C = [100 10 100]; %% best parameter  
else
    %% reserved for future use
end

%% bias
B = 1;

%% 7 for L2 regularized logistic regression (dual), 0 (for primal)
modelLearn = 0;

%% add test dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab'); 

filename = ['train' num2str(chooseSplit) '.txt'];
filePathTrain = strcat(fullPath, '/', filename);
[label_vector, instance_matrix] = libsvmread(filePathTrain);
[id d] = size(instance_matrix);
fprintf('finished reading train file\n');

%% Total number of possible labels
labelData = unique(sort(label_vector));

filename = ['fscore_TD1vsAll_' datasetFolderName '.txt'];
fullFileName = fullfile(fullPath, filename);
        
delete(fullFileName);
diary(fullFileName);

%% Timer variable
stopTime = 0;

%% Choose best parameter level wise
for nL = 1:noOfLevels
    %% Load the label info for each node (merged labels are in one line)
    groupMatrix = zeros(length(labelData), length(labelData));
    openFile = ['level' num2str(nL) datasetFolderName '.txt'];
    fid = fopen(openFile);
    tline = fgetl(fid);
    labelSet = labelData;
    nLine = 0;
    while ischar(tline)
         nLine = nLine + 1;
         splitLine = strread(tline,'%s','delimiter',' ');
         numElement = length(splitLine);
         groupMatrix(nLine, 1) = str2num(cell2mat(splitLine(1)));
         repItem = str2num(cell2mat(splitLine(1)));
         for numCategories = 2:numElement
             groupMatrix(nLine, numCategories) = str2num(cell2mat(splitLine(numCategories)));
             ind = find(labelSet == str2num(cell2mat(splitLine(numCategories))));
             labelSet(ind) = repItem;
         end
         tline = fgetl(fid);
    end
    fclose(fid);  
    
    %% All labels
    labelSet = unique(sort(labelSet));
    [noLabels noColLabels] = size(labelSet);
    for k = 1:length(label_vector)
            %% This is the class label to be used for the one vs all classification
            [r c] = find(groupMatrix == label_vector(k));
            if(isempty(r) || isempty(c))
                Y(k, 1) = -1;  
            else
                Y(k, 1) = groupMatrix(r, 1); 
            end
    end
        
    cmd = ['-c ', num2str(C(nL)), ' -s ', num2str(modelLearn), ' -B ', num2str(B)];

    %% train Model
    startTime = tic;
    model = ovrtrain(Y, instance_matrix, cmd);
    stopTime = stopTime + toc(startTime);
end
diary('off');
