%% This file is used to find the nodes that needs to be removed
%% Azad Naik

clc
clear all
close all

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'ng';

%% choose split number
chooseSplit = 0;

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;    
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    noOfLevels = 5;  
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;   
else
    %% reserved for future use
end

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% load the score for each node
filename = ['fscore_TD1vsAll_' datasetFolderName '.txt'];
fScore = load(filename);

% Load the leaf nodes index file
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    dataM = unique(label_vectorTrain);
    [NSizeDataM dSizeDataM] = size(dataM);
else
    filename = 'mapping.txt';
    dataM = load(filename);
    [NSizeDataM dSizeDataM] = size(dataM);
end

count = 0;
count1 = 0;

newLabelsInfo  = [];
for nL = 1:noOfLevels-1
    clear newLabelFile
    filename = ['level' num2str(nL) datasetFolderName 'label.txt'];
    labelFile = load(filename);
    [rlabelFile clabelFile] = size(labelFile);
    newLabelFile = [labelFile(:, 1) fScore(count+1:count+rlabelFile, 1)];
    count = count+rlabelFile;
    for i = 1:length(newLabelFile)
        [r c] = find(dataM == newLabelFile(i, 1));
        if(~isempty(r))
            newLabelFile(i, :) = 0;
        end
    end
    newLabelFile(~any(newLabelFile,2), :) = [];
    newLabelsInfo = [newLabelsInfo; newLabelFile];
    thresholdVal = mean(newLabelFile(:, 2)) + std(newLabelFile(:, 2));
    [r c] = find(newLabelFile(:, 2) > thresholdVal);
    for j = 1:length(r)
        count1 = count1 + 1;
        removalNode(count1, 1) = newLabelFile(r(j, 1), 1);
    end
end

%% Levelwise node removal
filename = 'levelinr_noderemoved.txt';
fullFileName = fullfile(fullPath, filename);
delete(fullFileName);
dlmwrite(fullFileName, sort(removalNode), 'precision', 6, 'delimiter', ' ');
[val Ind] = sort(newLabelsInfo(:, 2), 'descend');

%% Global node removal
if(strcmp(datasetFolderName, 'clef') || strcmp(datasetFolderName, 'ng')) 
	threshold = mean(newLabelsInfo(:, 2)) + 0.5*std(newLabelsInfo(:, 2));
	[r c] = find(newLabelsInfo(:, 2) > threshold);
    nodesToRemove = newLabelsInfo(r, 1);
else
	threshold = mean(newLabelsInfo(:, 2)) + std(newLabelsInfo(:, 2));
	[r c] = find(newLabelsInfo(:, 2) > threshold);
    nodesToRemove = newLabelsInfo(r, 1);
end
filename = 'globalinr_noderemoved.txt';
fullFileName = fullfile(fullPath, filename);
delete(fullFileName);
dlmwrite(fullFileName, sort(nodesToRemove), 'precision', 6, 'delimiter', ' ');
