%% Level file creation based on cat_hier_mlf file
%% Azad Naik

clc;
clear all;
close all;

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name
datasetFolderName = 'dmoz_2010_small';

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% flattening method to use
flatMethod = 'mlf';

%% choose split
chooseSplit = 0;

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5-2;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5-2;
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    noOfLevels = 5-2;    
else
end

% Load the hierarchy file
filename = ['cat_hier_' flatMethod '.txt'];
dataHierarchy = load(filename);
[NSizeDataH dSizeDataH] = size(dataHierarchy);

% Load the leaf nodes index file
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    dataM = unique(label_vectorTrain);
    [NSizeDataM dSizeDataM] = size(dataM);
else
    filename = 'mapping.txt';
    dataM = load(filename);
    [NSizeDataM dSizeDataM] = size(dataM);
end

%% contains leaf node to root info
chainMatrix = zeros(NSizeDataM, noOfLevels+1);
chainMatrix(:, 1) = dataM(:, 1);

for i = 1:NSizeDataM
    dataHierarchyLength = 1;
    toFind = dataM(i, 1);
    for j = 1:noOfLevels+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

rootNode = chainMatrix(1, noOfLevels+1);
modChainMatrix = chainMatrix;
for i = 1:NSizeDataM
    shift = 0;
    for j = noOfLevels:-1:1
        if(chainMatrix(i, j) == rootNode)
            shift = shift + 1;
        else
            break;
        end
    end
    if(shift > 0)
        modChainMatrix(i, 1:noOfLevels) = circshift(chainMatrix(i, 1:noOfLevels), [1 shift]);
    end
end

count = 0;
for i = noOfLevels:-1:1
    count = count + 1;
    filename = ['level' num2str(count) datasetFolderName '_' flatMethod '.txt'];
    fullFileName = fullfile(fullPath, filename);    
    levelInf = unique(modChainMatrix(:, i));
    removeNode = find(levelInf(:,:) == rootNode);
    levelInf(removeNode, 1) = NaN;
    levelInf = levelInf(isfinite(levelInf(:, 1)), 1);
    [levelRow levelCol] = size(levelInf);
    delete(fullFileName)
    %% Create leaf nodes file
    for j = 1:levelRow
        isLeaf = find(levelInf(j,1) == dataM);
        if(isempty(isLeaf))
            %% Not a leaf node
            checkRows = find(modChainMatrix(:, i) == levelInf(j, 1));
            valLeaf = chainMatrix(checkRows, 1);
            dlmwrite(fullFileName, valLeaf', 'precision',6, '-append', 'delimiter',' ');
        else
            dlmwrite(fullFileName, levelInf(j,1), 'precision',6, '-append', 'delimiter',' ');
        end
    end
end

display('pausing for 10 seconds (increase the pause if you get error)...');
pause(10); %% this gives enough time for the file to be written before accessing it

%% Store sorted flattened hierarchy
for nL = 1:noOfLevels
    
    %% Load the label info for each node (merged labels are in one line)
    openFile = ['level' num2str(nL) datasetFolderName '_' flatMethod '.txt'];
    fullFileName = fullfile(fullPath, openFile); 
    
    % open file
    fid = fopen(fullFileName);
    if (fid == -1)
        error(['Error Opening file', openFile]);
    end
    
    clear grpMatrix
    % start reading the file line by line
    id = 0;
    while (~feof(fid))
        id = id+1;
        grpMatrix{id} = fgetl(fid);
    end 
    % close file
    fclose(fid);
    fprintf('Finished Reading File\n');

    ctr = 0;
    clear labelData col
    for i = 1:id
        splitDMOZ = strread(grpMatrix{i}, '%s');
        if(~isempty(splitDMOZ))
            tempData = splitDMOZ(1);
            ctr = ctr + 1;
            labelData(ctr, 1) = str2num(cell2mat(tempData));
            labelData(ctr, 2) = i;
        end
    end
    
    [values, order] = sort(labelData(:, 1));
    sortedmatrix = labelData(order, :);

    clear grpMatrixMod
    for i = 1:length(labelData)
        grpMatrixMod{i} = grpMatrix{sortedmatrix(i, 2)};
    end
    
    delete(fullFileName);
    for i = 1:length(labelData)
        dlmwrite(fullFileName, grpMatrixMod{i}, 'precision',6, '-append', 'delimiter','');
    end
end