%% Creates new hierarchy after inconsistent node removal using Level-INR or Global-INR approach
%% Azad Naik

clc
clear all
close all

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'ng';

%% choose split number
chooseSplit = 0;

%% Proposed method to use
levelCutoff = 0; globalCutoff = 1;

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    noOfLevels = 5;    
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;
else
end

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

if(levelCutoff == 1) 
    inrMethod = 'levelinr';
elseif(globalCutoff == 1) 
    inrMethod = 'globalinr';
else
end

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

% Load the hierarchy file
filename = 'cat_hier.txt';
dataHierarchy = load(filename);
[NSizeDataH dSizeDataH] = size(dataHierarchy);

% Load the leaf nodes index file
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    dataM = unique(label_vectorTrain);
    [NSizeDataM dSizeDataM] = size(dataM);
else
    filename = 'mapping.txt';
    dataM = load(filename);
    [NSizeDataM dSizeDataM] = size(dataM);
end

% Load the leaf nodes index file
filename = [inrMethod '_noderemoved.txt'];
dataR = load(filename);
[NSizeDataR dSizeDataR] = size(dataR);

%% Modify the cat_hier by removing nodes line wise
for i = 1:NSizeDataR
    clear r c
    [r c] = find(dataHierarchy(:, 2) == dataR(i, 1));
    pRemovedNode = dataHierarchy(r, 1);
    dataHierarchy(r, :) = 0;
    [r c] = find(dataHierarchy(:, 1) == dataR(i, 1));
    dataHierarchy(r, 1) = pRemovedNode;
end

dataHierarchy(~any(dataHierarchy,2), :) = [];   % remove rows containing only 0 
filename = ['cat_hier_' inrMethod '.txt'];
fullFileName = fullfile(fullPath, filename);
delete(fullFileName);
dlmwrite(fullFileName, dataHierarchy, 'precision', 6, 'delimiter', ' ');
