%% flattens the hierarchy and creates new hierarchy file
%% Azad Naik 

clc;
clear all;
close all;

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name
datasetFolderName = 'clef';

%% flattening method to use
flatMethod = 'tlf';

%% choose split
chooseSplit = 0;

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    heightHierarchy = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    heightHierarchy = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    heightHierarchy = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    heightHierarchy = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    heightHierarchy = 5;
elseif(strcmp(datasetFolderName, 'diatoms'))
    heightHierarchy = 3;
else
end

% Load the hierarchy file
filename = 'cat_hier.txt';
dataHierarchy = load(filename);
[NSizeDataH dSizeDataH] = size(dataHierarchy);

% Load the leaf nodes index file
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    dataM = unique(label_vectorTrain);
    [NSizeDataM dSizeDataM] = size(dataM);
else
    filename = 'mapping.txt';
    dataM = load(filename);
    [NSizeDataM dSizeDataM] = size(dataM);
end


%% Modify the cat_hier by removing last level information
%% Create the group matrix for the leaf nodes (contains leaf node to root info)
chainMatrix = zeros(NSizeDataM, heightHierarchy+1);
chainMatrix(:, 1) = dataM(:, 1);

for i = 1:NSizeDataM
    dataHierarchyLength = 1;
    toFind = dataM(i, 1);
    for j = 1:heightHierarchy+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

if (strcmp(flatMethod, 'tlf')) %% top level flattening
    for i = 1:NSizeDataM
        clear a b
        [a b] = find(chainMatrix(i, :) > 0);
        labelInfo(i, 1) = chainMatrix(i, b(end));
    end
    labelInfo = unique(labelInfo);

    ctr = 0;
    for i = 1:length(labelInfo)
        clear r c
        [r c] = find(labelInfo(i, 1) == dataM);
        if(r > 0)
            labelInfo(i) = 0;
        end
    end
    labelInfo = unique(labelInfo(find(labelInfo > 0)));

    ctr = 0;
    for i = 1:length(labelInfo)
        clear r c
        [r c] = find(labelInfo(i) == dataHierarchy(:, 2));
        if(~isempty(r))
            dataHierarchy(r, :) = 0;
            labelInfo(i, 2) = dataHierarchy(r, 1);
        end
    end
    dataHierarchy(~any(dataHierarchy, 2), :) = []; 
    for i = 1:size(labelInfo, 1)
        dataHierarchy(dataHierarchy(:, 1) == labelInfo(i, 1), 1) = labelInfo(i, 2); 
    end
elseif (strcmp(flatMethod, 'blf')) %%  Bottom level flattening
    for i = 1:NSizeDataM
        clear a b
        [a b] = find(chainMatrix(i, :) > 0);
        labelInfo(i, 1) = chainMatrix(i, b(max(1, end-heightHierarchy+2)));
    end
    labelInfo = unique(labelInfo);

    for i = 1:NSizeDataM
        clear a b
        [a b] = find(chainMatrix(i, :) > 0);
        labelInfo(i, 1) = chainMatrix(i, b(max(1, end-heightHierarchy+2)));
    end
    labelInfo = unique(labelInfo);
    
    %% remove labels from labelInfo
    ctr = 0;
    for i = 1:length(labelInfo)
        [r c] = find(labelInfo(i, 1) == dataM);
        if(r > 0)
            labelInfo(i) = 0;
        end
    end
    labelInfo = unique(labelInfo(find(labelInfo > 0)));
    ctr = 0;
    for i = 1:length(labelInfo)
        clear r c
        [r c] = find(labelInfo(i) == dataHierarchy(:, 2));
        if(~isempty(r))
            labelInfo(i, 2) = dataHierarchy(r, 1);
            dataHierarchy(r, :) = 0;
        end
    end
    dataHierarchy(~any(dataHierarchy, 2), :) = []; 
    for i = 1:size(labelInfo, 1)
        dataHierarchy(dataHierarchy(:, 1) == labelInfo(i, 1), 1) = labelInfo(i, 2); 
    end
else
    
end

filename = ['cat_hier_' flatMethod '.txt'];
fullFileName = fullfile(fullPath, filename);
delete(fullFileName);
fid = fopen(fullFileName,'wt');
for i = 1:size(dataHierarchy, 1)
    fprintf(fid,'%d\t%d\n',dataHierarchy(i, :));
end

fclose(fid);
dlmwrite(fullFileName, dataHierarchy, 'precision', 6, 'delimiter',' ');


