%% creates the node label corresponding to each node

clc
clear all
close all

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'ng';

%% choose split number
chooseSplit = 0;

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    heightHierarchy = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    heightHierarchy = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    heightHierarchy = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    heightHierarchy = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    heightHierarchy = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    heightHierarchy = 5;
elseif(strcmp(datasetFolderName, 'diatoms'))
    heightHierarchy = 3;    
else
    %% reserved for future use
end

% Load the hierarchy file
filename = 'cat_hier.txt';
dataHierarchy = load(filename);
[NSizeDataH dSizeDataH] = size(dataHierarchy);

% Load the leaf nodes index file
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    dataM = unique(label_vectorTrain);
    [NSizeDataM dSizeDataM] = size(dataM);
else
    filename = 'mapping.txt';
    dataM = load(filename);
    [NSizeDataM dSizeDataM] = size(dataM);
end

%% contains leaf node to root info
chainMatrix = zeros(NSizeDataM, heightHierarchy+1);
chainMatrix(:, 1) = dataM(:, 1);

for i = 1:NSizeDataM
    dataHierarchyLength = 1;
    toFind = dataM(i, 1);
    for j = 1:heightHierarchy+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

filename = ['train' num2str(chooseSplit) '.txt'];
filePathTrain = strcat(fullPath, '/', filename);
[label_vector, instance_matrix] = libsvmread(filePathTrain);
[id d] = size(instance_matrix);
fprintf('Finished Reading train File\n');

for nL = 1:heightHierarchy
    %% Load the label info for each node (merged labels are in one line)
    openFile = ['level' num2str(nL) datasetFolderName '.txt'];
    
    % open file
    fid = fopen(openFile);
    if (fid == -1)
    error(['Error Opening file', fileName]);
    end

    % start reading the file line by line
    id = 0;
    while (~feof(fid))
        id = id+1;
        grpMatrix{id} = fgetl(fid);
    end 
    % close file
    fclose(fid);
    fprintf('Finished Reading File\n');
    
    clear groupMatrix
    fid = fopen(openFile);
    tline = fgetl(fid);
    nLine = 0;
    while ischar(tline)
         nLine = nLine + 1;
         splitLine = strread(tline,'%s','delimiter',' ');
         numElement = length(splitLine);
         groupMatrix(nLine, 1) = str2num(cell2mat(splitLine(1)));
         repItem = str2num(cell2mat(splitLine(1)));
         labelSet(nLine, 1) = repItem;
         for numCategories = 2:numElement
             %% creates group matrix
             groupMatrix(nLine, numCategories) = str2num(cell2mat(splitLine(numCategories)));
         end
         tline = fgetl(fid);
    end
    fclose(fid);      
    groupMatrix(~any(groupMatrix,2), :) = [];   % remove rows containing only 0 
    groupMatrix(:, ~any(groupMatrix, 1)) = [];  % remove all columns containing only 0

    clear nodeLabel
    nodeLabel = zeros(id, 2);
    for i = 1:id
        splitDMOZ = strread(grpMatrix{i}, '%s');
        tempData = splitDMOZ(1);
        labelData(i, 1) = str2num(cell2mat(tempData));
        nodeLabel(i, 1) = chainMatrix(find(chainMatrix(:, 1) == labelData(i, 1)), max(1, length(find(chainMatrix(find(chainMatrix(:, 1) == labelData(i, 1)),:) > 0))-nL+1));
    end
    
    for i = 1:length(label_vector)
        [r c] = find(groupMatrix == label_vector(i,1));
        nodeLabel(r, 2) = nodeLabel(r, 2) + 1;
    end
    
    openFile1 = ['level' num2str(nL) datasetFolderName 'label.txt'];
    fullFileName = fullfile(fullPath, openFile1);
    delete(fullFileName);
    dlmwrite(fullFileName, nodeLabel, 'precision', 6, 'delimiter', ' ');

end
