%% Reference:
%%  http://datamin.ubbcluj.ro/wiki/index.php/Evaluation_methods_in_text_categorization
%%  Text Categorization with Class-Based and Corpus-Based Keyword Selection by Arzucan Ozgur, Levent Ozgur, and Tunga Gungor
function [microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(realIndex, predictedIndex, distinctClassLabels)

    [N d] = size(realIndex);
    [noOfClass dclass] = size(distinctClassLabels);
%     matlabpool(3);
    microCalMatrix = zeros(noOfClass, 4); %% TP, FP, FN, TN
    sumMacroPrecision = 0;
    sumMacroRecall = 0;
    sumMacroF1 = 0;
    sumMCCscore = 0;
    for i = 1:noOfClass
        macroCalMatrix = zeros(1, 4);
        for j = 1:N
            %% TP
            if((realIndex(j, 1) == distinctClassLabels(i, 1)) && (predictedIndex(j, 1) == distinctClassLabels(i, 1))) 
                microCalMatrix(i, 1) = microCalMatrix(i, 1) + 1;
                macroCalMatrix(1, 1) = macroCalMatrix(1, 1) + 1;
            %% TN
            elseif((realIndex(j, 1) ~= distinctClassLabels(i, 1)) && (predictedIndex(j, 1) ~= distinctClassLabels(i, 1)))
                microCalMatrix(i, 4) = microCalMatrix(i, 4) + 1;
                macroCalMatrix(1, 4) = macroCalMatrix(1, 4) + 1;
            %% FP
            elseif((realIndex(j, 1) ~= distinctClassLabels(i, 1)) && (predictedIndex(j, 1) == distinctClassLabels(i, 1)))
                microCalMatrix(i, 2) = microCalMatrix(i, 2) + 1;
                macroCalMatrix(1, 2) = macroCalMatrix(1, 2) + 1;
            %% FN
            else
                microCalMatrix(i, 3) = microCalMatrix(i, 3) + 1;
                macroCalMatrix(1, 3) = macroCalMatrix(1, 3) + 1;
            end
        end
        if(macroCalMatrix(1, 1) > 0)
            macroPrecision = (macroCalMatrix(1, 1)/(macroCalMatrix(1, 1) + macroCalMatrix(1, 2)));
            macroRecall = (macroCalMatrix(1, 1)/(macroCalMatrix(1, 1) + macroCalMatrix(1, 3)));
        else
            macroPrecision = 0;
            macroRecall = 0;
        end
        sumMacroPrecision = sumMacroPrecision + macroPrecision;
        sumMacroRecall = sumMacroRecall + macroRecall;
        if((macroPrecision+macroRecall) > 0)
            sumMacroF1 = sumMacroF1 + (2*macroPrecision*macroRecall)/(macroPrecision+macroRecall);
        else
            sumMacroF1 = sumMacroF1;
        end
        if(((microCalMatrix(i, 1)+microCalMatrix(i, 2))*(microCalMatrix(i, 1)+microCalMatrix(i, 3))*(microCalMatrix(i, 4)+microCalMatrix(i, 2))*(microCalMatrix(i, 4)+microCalMatrix(i, 3))) > 0)
            sumMCCscore = sumMCCscore + (((microCalMatrix(i, 1)*microCalMatrix(i, 4))-(microCalMatrix(i, 2)*microCalMatrix(i, 3)))/(sqrt((microCalMatrix(i, 1)+microCalMatrix(i, 2))*(microCalMatrix(i, 1)+microCalMatrix(i, 3))*(microCalMatrix(i, 4)+microCalMatrix(i, 2))*(microCalMatrix(i, 4)+microCalMatrix(i, 3)))));
        else
            sumMCCscore = sumMCCscore;
        end
    end
%     matlabpool close;
    TPSum = sum(microCalMatrix(:, 1));
    FPSum = sum(microCalMatrix(:, 2));
    FNSum = sum(microCalMatrix(:, 3));
    microPrecisionVal = TPSum/(TPSum+FPSum);
    microRecallVal = TPSum/(TPSum+FNSum);
    macroPrecisionVal = sumMacroPrecision/noOfClass;
    macroRecallVal = sumMacroRecall/noOfClass;
    macroF1Val = sumMacroF1/noOfClass;
    microF1Val = (2*microPrecisionVal*microRecallVal)/(microPrecisionVal+microRecallVal);
    MCCVal = sumMCCscore/noOfClass;
end
    