%% Result on training examples with < 5 examples
%% Azad Naik

clc
clear all
close all

%% threshold on numExamples
numExample = 10;
minimumExample = 5;

%% modelName 
modelName = 'globalinr';

%% Dataset path
datasetPath = '../../data/';

%% Result path
resultPath = '../../result/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'dmoz_2010_small';

if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;    
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;
else
end

%% add dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add result path
fullResultPath = strcat(resultPath, datasetFolderName);
addpath(fullResultPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% load training file in liblinear format 
filename = 'train.txt';
filePathTrain = strcat(fullPath, '/', filename);
[label_vector, instance_matrix] = libsvmread(filePathTrain);
[id d] = size(instance_matrix);
fprintf('Finished Reading train File\n');

%% Total number of possible labels
labelData = unique(sort(label_vector));

%% count
for i = 1:length(labelData)
    labelData(i, 2) = length(find(label_vector == labelData(i, 1)));
end

[r c] = find(labelData(:, 2) <= numExample & labelData(:, 2) > minimumExample);
selectedLabel = labelData(r, 1);

%% load test file in liblinear format 
filename = 'test.txt';
filePathTest = strcat(fullPath, '/', filename);
[label_vectorT, instance_matrixT] = libsvmread(filePathTest);
[idT dT] = size(instance_matrixT);
fprintf('Finished Reading test File\n');

testExCheck = 0;
count = 0;
for i = 1:length(label_vectorT)
    clear r c
    [r c] = find(selectedLabel(:, 1) == label_vectorT(i, 1));
    if(~isempty(r)) %% add row
        count = count + 1;
        testExCheck(count, 1) = i;
    end
end

%% save label number of test file
filename = ['test_row_less_than_5_example_' datasetFolderName '.txt'];
fullFileName = fullfile(fullPath, filename);
dlmwrite(fullFileName, testExCheck, 'precision', '%d');  

actualLabel = label_vectorT(testExCheck, 1);
if(strcmp(modelName, 'td'))
    TDmodelFilename = ['test_prediction_' datasetFolderName '.txt'];
else
    TDmodelFilename = ['test_prediction_' datasetFolderName '_' modelName '.txt'];
end
FlatmodelFilename = ['test_prediction_' datasetFolderName '_flat.txt'];
TDpredicted = load(TDmodelFilename);
Flatpredicted = load(FlatmodelFilename);

TDpredictedLabel = TDpredicted(testExCheck, end);
FlatpredictedLabel = Flatpredicted(testExCheck, end);

[microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(actualLabel, TDpredictedLabel, selectedLabel);
microF1Val
macroF1Val

%% Load Hierarchy File [EdgeNode1, EdgeNode2]
dataHierarchy = load('cat_hier.txt');
[Nh dh] = size(dataHierarchy);

leafNodes = size(Nh, 2);
leafNodes = dataHierarchy(:, 2);

leafNodes = sort(leafNodes);
leafNodes = unique(leafNodes);
[Nleaf dLeaf] = size(leafNodes);

for i = 1:Nleaf
    a = find(dataHierarchy(:, 1) == leafNodes(i, 1));
    if(size(a, 1) > 0)
        %% Internal Node 
        leafNodes(i, 2) = 0;
    else
        %% Leaf Node
        leafNodes(i, 2) = 1;
    end
end

[r, c, v] = find(leafNodes(:, 2) == 1);
leafNodes = leafNodes(r, 1);
[Nleaf dLeaf] = size(leafNodes);


%% contains leaf node to root info
chainMatrix = zeros(Nleaf, noOfLevels+1);
chainMatrix(:, 1) = leafNodes(:, 1);

for i = 1:Nleaf
    dataHierarchyLength = 1;
    toFind = leafNodes(i, 1);
    for j = 1:noOfLevels+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

%% Load Label Data File [Actual Label, Predicted Label]
dataLabel = [actualLabel TDpredictedLabel]; 
[Nl dl] = size(dataLabel);

%% predict Hierarchical Precision/Recall
hPDenominator = 0; hPRNumerator = 0;
hRDenominator = 0; 
for i = 1:Nl
    %% True Label
    a = find(chainMatrix(:, 1) == dataLabel(i, 1));
    %% Predicted Label
    b = find(chainMatrix(:, 1) == dataLabel(i, 2));
    sizehP = size(find(chainMatrix(b, :) > 0), 2);
    sizehR = size(find(chainMatrix(a, :) > 0), 2);
    hPDenominator = hPDenominator + sizehP;
    hRDenominator = hRDenominator + sizehR;
    for j = 1:sizehP
        hPRNumerator = hPRNumerator + size(find(ismember(chainMatrix(a, :), chainMatrix(b, j)) > 0), 2);
    end  
    hPRNumerator = hPRNumerator;
end
 
hierarchicalPrecision = hPRNumerator/hPDenominator;
hierarchicalRecall = hPRNumerator/hRDenominator;
hierarchicalF1 = (2*hierarchicalPrecision*hierarchicalRecall)/(hierarchicalPrecision + hierarchicalRecall)

% %% Tree-induced Error
% TE = 0;
% for i = 1:Nl
%     if(dataLabel(i, 1) == dataLabel(i, 2))
%         %% No Error
%         TE = TE + 0;
%     else
%         %% True Label
%         a = find(chainMatrix(:, 1) == dataLabel(i, 1));
%         sizea = size(find(chainMatrix(a, :) > 0), 2);
%         %% Predicted Label
%         b = find(chainMatrix(:, 1) == dataLabel(i, 2));
%         sizeb = size(find(chainMatrix(b, :) > 0), 2);
%         foundFlag = 0;
%         for j = 1:sizea
%            found = size(find(ismember(chainMatrix(a, j), chainMatrix(b, :)) > 0), 2); 
%            if(found > 0)
%                %%found
%                TE = TE + (j-1) + (find(chainMatrix(b, :) == chainMatrix(a, j)) - 1);
%                foundFlag = 1;
%                break;
%            else
%            end    
%         end
%         if(foundFlag == 0)
%             TE = TE + sizea + sizeb;
%         end
%     end
% end
% 
% TE = TE/Nl

[microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(actualLabel, FlatpredictedLabel, selectedLabel);
microF1Val
macroF1Val

%% Load Hierarchy File [EdgeNode1, EdgeNode2]
dataHierarchy = load('cat_hier.txt');
[Nh dh] = size(dataHierarchy);

leafNodes = size(Nh, 2);
leafNodes = dataHierarchy(:, 2);

leafNodes = sort(leafNodes);
leafNodes = unique(leafNodes);
[Nleaf dLeaf] = size(leafNodes);

for i = 1:Nleaf
    a = find(dataHierarchy(:, 1) == leafNodes(i, 1));
    if(size(a, 1) > 0)
        %% Internal Node 
        leafNodes(i, 2) = 0;
    else
        %% Leaf Node
        leafNodes(i, 2) = 1;
    end
end

[r, c, v] = find(leafNodes(:, 2) == 1);
leafNodes = leafNodes(r, 1);
[Nleaf dLeaf] = size(leafNodes);


%% contains leaf node to root info
chainMatrix = zeros(Nleaf, noOfLevels+1);
chainMatrix(:, 1) = leafNodes(:, 1);

for i = 1:Nleaf
    dataHierarchyLength = 1;
    toFind = leafNodes(i, 1);
    for j = 1:noOfLevels+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

%% Load Label Data File [Actual Label, Predicted Label]
dataLabel = [actualLabel FlatpredictedLabel]; 
[Nl dl] = size(dataLabel);

%% predict Hierarchical Precision/Recall
hPDenominator = 0; hPRNumerator = 0;
hRDenominator = 0; 
for i = 1:Nl
    %% True Label
    a = find(chainMatrix(:, 1) == dataLabel(i, 1));
    %% Predicted Label
    b = find(chainMatrix(:, 1) == dataLabel(i, 2));
    sizehP = size(find(chainMatrix(b, :) > 0), 2);
    sizehR = size(find(chainMatrix(a, :) > 0), 2);
    hPDenominator = hPDenominator + sizehP;
    hRDenominator = hRDenominator + sizehR;
    for j = 1:sizehP
        hPRNumerator = hPRNumerator + size(find(ismember(chainMatrix(a, :), chainMatrix(b, j)) > 0), 2);
    end  
    hPRNumerator = hPRNumerator;
end
 
hierarchicalPrecision = hPRNumerator/hPDenominator;
hierarchicalRecall = hPRNumerator/hRDenominator;
hierarchicalF1 = (2*hierarchicalPrecision*hierarchicalRecall)/(hierarchicalPrecision + hierarchicalRecall)

% %% Tree-induced Error
% TE = 0;
% for i = 1:Nl
%     if(dataLabel(i, 1) == dataLabel(i, 2))
%         %% No Error
%         TE = TE + 0;
%     else
%         %% True Label
%         a = find(chainMatrix(:, 1) == dataLabel(i, 1));
%         sizea = size(find(chainMatrix(a, :) > 0), 2);
%         %% Predicted Label
%         b = find(chainMatrix(:, 1) == dataLabel(i, 2));
%         sizeb = size(find(chainMatrix(b, :) > 0), 2);
%         foundFlag = 0;
%         for j = 1:sizea
%            found = size(find(ismember(chainMatrix(a, j), chainMatrix(b, :)) > 0), 2); 
%            if(found > 0)
%                %%found
%                TE = TE + (j-1) + (find(chainMatrix(b, :) == chainMatrix(a, j)) - 1);
%                foundFlag = 1;
%                break;
%            else
%            end    
%         end
%         if(foundFlag == 0)
%             TE = TE + sizea + sizeb;
%         end
%     end
% end
% 
% TE = TE/Nl