%% one vs all; flat/hierarchical measures computation
%% Azad Naik

clc
clear all
close all

%% Result path
resultPath = '../../result/';

%% Test dataset path
datasetPath = '../../data/';

%% Dataset folder name
datasetFolderName = 'ng';

%% choose split number
chooseSplit = 0;

%% Proposed method to use (levelinr or globalinr)
inrMethod = 'globalinr';

%% original hierachy (1 for original hierarchy, 0 for new hierarchy)
originalHierarchy = 1;

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;
else
    %% do nothing
end

%% bias
B = 1;

%% 7 for L2 regularized logistic regression (dual), 0 (for primal)
modelLearn = 0;

%% add test dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% load Test file
filenameT = 'test.txt';
filePathTest = strcat(fullPath, '/', filenameT);
[label_vectorT, instance_matrixT] = libsvmread(filePathTest);
[idT dT] = size(instance_matrixT);
fprintf('Finished Reading test File\n');

%% correct label
realIndex = label_vectorT;

%% add result path
fullPath = strcat(resultPath, datasetFolderName);
addpath(fullPath);

% Each level prediction is stored in predictedIndex variable last columns is true index
predictedIndex = -ones(idT, noOfLevels);

%% Load label file (mapping.txt file contains all leaf node labels)
if(strcmp(datasetFolderName, 'diatoms'))
    filename = ['train' num2str(chooseSplit) '.txt'];
    filePathTrain = strcat(fullPath, '/', filename);
    [label_vectorTrain, instance_matrixTrain] = libsvmread(filePathTrain);
    labelSet = unique(label_vectorTrain);
else
    filename = 'mapping.txt';
    labelSet = load(filename);
    labelSet = sort(labelSet);
end

%% Timer
stopTime = 0;

for nL = 1:noOfLevels   
    clear model
    % Load the file
    filename = [num2str(chooseSplit) 'td_one_vs_all_' datasetFolderName '_level_' num2str(nL) '_' inrMethod '.mat'];
    model = load(filename);
    
    %% Load the label info for each node (merged labels are in one line)
    openFile = ['level' num2str(nL) datasetFolderName '_' inrMethod '.txt'];
    fid = fopen(openFile);
    tline = fgetl(fid);
    labelData = zeros(length(labelSet), 1); 
    nLine = 0;
    clear groupMatrix
    while ischar(tline)
         nLine = nLine + 1;
         splitLine = strread(tline,'%s','delimiter',' ');
         numElement = length(splitLine);
         groupMatrix(nLine, 1) = str2num(cell2mat(splitLine(1)));
         labelData(nLine) = groupMatrix(nLine, 1);
         repItem = str2num(cell2mat(splitLine(1)));
         for numCategories = 2:numElement
             groupMatrix(nLine, numCategories) = str2num(cell2mat(splitLine(numCategories)));
             ind = find(labelData == str2num(cell2mat(splitLine(numCategories))));
             labelData(ind) = repItem;
         end
         tline = fgetl(fid);
    end
    fclose(fid);
    labelData = unique(labelData(1:nLine, :));

    if(nL > 1)
        openFile = ['level' num2str(nL-1) datasetFolderName '_' inrMethod '.txt'];
        fid1 = fopen(openFile);
        tline = fgetl(fid);
        aData1 = zeros(length(labelSet), 1); 
        nLine = 0;
        clear groupMatrix1
        while ischar(tline)
             nLine = nLine + 1;
             splitLine = strread(tline,'%s','delimiter',' ');
             numElement = length(splitLine);
             groupMatrix1(nLine, 1) = str2num(cell2mat(splitLine(1)));
             aData1(nLine) = groupMatrix1(nLine, 1);
             repItem = str2num(cell2mat(splitLine(1)));
             for numCategories = 2:numElement
                 groupMatrix1(nLine, numCategories) = str2num(cell2mat(splitLine(numCategories)));
                 ind = find(aData1 == str2num(cell2mat(splitLine(numCategories))));
                 aData1(ind) = repItem;
             end
             tline = fgetl(fid);
        end
        fclose(fid1); 
        aData1 = unique(aData1(1:nLine, :));
    end
    
    %% Top down label assignment
%     for k = 1:length(label_vectorT)   
%         [r c] = find(groupMatrix == label_vectorT(k));
%         if(isempty(r) || isempty(c))
%             Y(k, 1) = -1;
%         else
%             Y(k, 1) = groupMatrix(r, 1);
%         end
%     end
    Y = zeros(length(label_vectorT), 1);
      
    if(nL == 1)
        startTime = tic;
        [pred ac decv] = ovrpredict(Y, instance_matrixT, model);
        stopTime = stopTime + toc(startTime);
        predictedIndex(:, nL) = pred;
    else 
        for nodeLevel = 1:length(aData1)
            %% all rows with predicted label
            clear rP cP
            [rP cP] = find(aData1(nodeLevel, 1) == predictedIndex(:, nL-1));
            [rowLabel colLabel] = size(find(groupMatrix1(nodeLevel, :) > 0));
            if(colLabel == 1)
                predictedIndex(rP, nL:noOfLevels) = aData1(nodeLevel, 1);
            else
                rowMatrixStore = zeros(colLabel, 1);
                for iterLabel = 1:colLabel
                    [r1 c1] = find(groupMatrix == groupMatrix1(nodeLevel, iterLabel));
                    rowMatrixStore(iterLabel, 1) = r1;
                end
                rowMatrixStore = unique(rowMatrixStore);
                %% new model
                models = cell(length(rowMatrixStore),1);

                for j = 1:length(rowMatrixStore)
                    newModels{j, 1} = model.models{rowMatrixStore(j, 1)};
                end
                %% new model
                modelNew = struct('models', {newModels}, 'labelSet', labelData(rowMatrixStore, 1));
                
                startTime = tic;
                if(length(rP) > 0)
                    [pred ac decv] = ovrpredict(Y(rP, 1), instance_matrixT(rP, :), modelNew);                
                    predictedIndex(rP, nL) = pred;
                end
                stopTime = stopTime + toc(startTime);
                
            end
        end
    end
end

[microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(label_vectorT, predictedIndex(:, noOfLevels), labelSet(:, 1));
microF1Val
macroF1Val
 
%% Load Hierarchy File [EdgeNode1, EdgeNode2]
if(originalHierarchy == 1)
    hierName = 'cat_hier.txt';
else
    hierName = ['cat_hier_' inrMethod '.txt'];
end

%% Load Hierarchy File [EdgeNode1, EdgeNode2]
dataHierarchy = load(hierName);
[Nh dh] = size(dataHierarchy);

leafNodes = size(Nh, 2);
leafNodes = dataHierarchy(:, 2);

leafNodes = sort(leafNodes);
leafNodes = unique(leafNodes);
[Nleaf dLeaf] = size(leafNodes);

for i = 1:Nleaf
    a = find(dataHierarchy(:, 1) == leafNodes(i, 1));
    if(size(a, 1) > 0)
        %% Internal Node 
        leafNodes(i, 2) = 0;
    else
        %% Leaf Node
        leafNodes(i, 2) = 1;
    end
end

[r, c, v] = find(leafNodes(:, 2) == 1);
leafNodes = leafNodes(r, 1);
[Nleaf dLeaf] = size(leafNodes);


%% contains leaf node to root info
chainMatrix = zeros(Nleaf, noOfLevels+1);
chainMatrix(:, 1) = leafNodes(:, 1);

for i = 1:Nleaf
    dataHierarchyLength = 1;
    toFind = leafNodes(i, 1);
    for j = 1:noOfLevels+1
        %% gives index
        a = find(dataHierarchy(:, 2) == toFind);
        if(size(a, 1) == 0)
            break;
        else
            dataHierarchyLength = dataHierarchyLength + 1;
            chainMatrix(i, dataHierarchyLength) = dataHierarchy(a, 1);
            toFind = dataHierarchy(a, 1);
        end
    end
end

%% Load Label Data File [Actual Label, Predicted Label]
dataLabel = [label_vectorT predictedIndex(:, noOfLevels)]; 
[Nl dl] = size(dataLabel);

%% predict Hierarchical Precision/Recall
hPDenominator = 0; hPRNumerator = 0;
hRDenominator = 0; 
for i = 1:Nl
    %% True Label
    a = find(chainMatrix(:, 1) == dataLabel(i, 1));
    %% Predicted Label
    b = find(chainMatrix(:, 1) == dataLabel(i, 2));
    sizehP = size(find(chainMatrix(b, :) > 0), 2);
    sizehR = size(find(chainMatrix(a, :) > 0), 2);
    hPDenominator = hPDenominator + sizehP;
    hRDenominator = hRDenominator + sizehR;
    for j = 1:sizehP
        hPRNumerator = hPRNumerator + size(find(ismember(chainMatrix(a, :), chainMatrix(b, j)) > 0), 2);
    end  
    hPRNumerator = hPRNumerator;
end
 
hierarchicalPrecision = hPRNumerator/hPDenominator
hierarchicalRecall = hPRNumerator/hRDenominator
hierarchicalF1 = (2*hierarchicalPrecision*hierarchicalRecall)/(hierarchicalPrecision + hierarchicalRecall)

%% Tree-induced Error
TE = 0;
for i = 1:Nl
    if(dataLabel(i, 1) == dataLabel(i, 2))
        %% No Error
        TE = TE + 0;
    else
        %% True Label
        a = find(chainMatrix(:, 1) == dataLabel(i, 1));
        sizea = size(find(chainMatrix(a, :) > 0), 2);
        %% Predicted Label
        b = find(chainMatrix(:, 1) == dataLabel(i, 2));
        sizeb = size(find(chainMatrix(b, :) > 0), 2);
        foundFlag = 0;
        for j = 1:sizea
           found = size(find(ismember(chainMatrix(a, j), chainMatrix(b, :)) > 0), 2); 
           if(found > 0)
               %%found
               TE = TE + (j-1) + (find(chainMatrix(b, :) == chainMatrix(a, j)) - 1);
               foundFlag = 1;
               break;
           else
           end    
        end
        if(foundFlag == 0)
            TE = TE + sizea + sizeb;
        end
    end
end

TE = TE/Nl

sprintf('time taken for test dataset prediction: %0.5f mins', stopTime/60)

%% save predicted label
filename = [num2str(chooseSplit) 'test_prediction_' datasetFolderName '_' inrMethod '.txt'];
fullFileName = fullfile(fullPath, filename);
dlmwrite(fullFileName, predictedIndex, 'precision', '%d');  