%% one vs all model training
%% Azad Naik

clc
clear all
close all

%% Dataset path
datasetPath = '../../data/';

%% Dataset Folder Name; based on the dataset to be used it needs to be modified
datasetFolderName = 'clef';

%% choose split number
chooseSplit = 0;

%% User parameter Regularization C, Bias B
if(strcmp(datasetFolderName, 'clef'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ng'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'ipc'))
    noOfLevels = 3;
elseif(strcmp(datasetFolderName, 'dmoz_2010_small'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2010'))
    noOfLevels = 5;
elseif(strcmp(datasetFolderName, 'dmoz_2012'))
    noOfLevels = 5;    
elseif(strcmp(datasetFolderName, 'diatoms'))
    noOfLevels = 3;
else
end

%% User parameter Regularization C, Bias B
C = [0.001 0.01 0.1 1 10 100 1000];
B = 1;

%% 7 for L2 regularized logistic regression (dual), 0 (for primal)
modelLearn = 0;

%% Full dataset path
fullPath = strcat(datasetPath, datasetFolderName);
addpath(fullPath);

%% add path for modified liblinear package
addpath('../liblinear-1.94/matlab');

%% Select path based on Operating System
if ispc
    partialPath = '../../result/';
elseif isunix
	partialPath = '/home/anaik3/code/TD/result/';
else
    % other operating system
end 

%% Choose output folder name
resultPath = strcat(partialPath, datasetFolderName);
fn = fullfile(resultPath);
if exist(fn, 'dir')
   warning('directory exist- do you want to overwrite?');
   prompt = 'enter 1 for yes; any other key for no \n';
   userChoice = input(prompt);
   if (userChoice ~= 1)
       display('please choose different directory or overwrite existing directory...exiting program');
       return;
   end
   display('overwriting existing folder');
else
   mkdir(fn);
end
  
%% load training file in liblinear format 
filename = ['train' num2str(chooseSplit) '.txt'];
filePathTrain = strcat(fullPath, '/', filename);
[label_vector, instance_matrix] = libsvmread(filePathTrain);
[id d] = size(instance_matrix);
fprintf('finished reading train File\n');

%% load validation file in liblinear format 
filenameT = ['validation' num2str(chooseSplit) '.txt'];
filePathTest = strcat(fullPath, '/', filenameT);
[label_vectorT, instance_matrixT] = libsvmread(filePathTest);
fprintf('finished reading validation file\n');

%% Total number of possible labels
labelData = unique(sort(label_vector));

%% Timer variable
stopTime = 0;

%% Choose best parameter for the node
for nL = 1:noOfLevels
    %% Load the label info for each node (merged labels are in one line)
    groupMatrix = zeros(length(labelData), length(labelData));
    openFile = ['level' num2str(nL) datasetFolderName '_rewire.txt'];
    fid = fopen(openFile);
    tline = fgetl(fid);
    labelSet = labelData;
    nLine = 0;
    while ischar(tline)
         nLine = nLine + 1;
         splitLine = strread(tline,'%s','delimiter',' ');
         numElement = length(splitLine);
         groupMatrix(nLine, 1) = str2num(cell2mat(splitLine(1)));
         repItem = str2num(cell2mat(splitLine(1)));
         for numCategories = 2:numElement
             groupMatrix(nLine, numCategories) = str2num(cell2mat(splitLine(numCategories)));
             ind = find(labelSet == str2num(cell2mat(splitLine(numCategories))));
             labelSet(ind) = repItem;
         end
         tline = fgetl(fid);
    end
    fclose(fid);  
    
    %% All labels
    labelSet = unique(sort(labelSet));
    [noLabels noColLabels] = size(labelSet);
    for k = 1:length(label_vector)
            %% This is the class label to be used for the one vs all classification
            [r c] = find(groupMatrix == label_vector(k));
            if(isempty(r) || isempty(c))
                Y(k, 1) = -1;  
            else
                Y(k, 1) = groupMatrix(r, 1); 
            end
    end
    
    for k = 1:length(label_vectorT)
            %% This is the class label to be used for the one vs all classification
            [r c] = find(groupMatrix == label_vectorT(k));
            if(isempty(r) || isempty(c))
                YT(k, 1) = -1;  
            else
                YT(k, 1) = groupMatrix(r, 1);
            end
    end
      
    %% best model with best macroF1
    bestModel = 0;
    bestMacroF1 = 0;
    
    for regTest = 1:length(C)
        cmd = ['-c ', num2str(C(regTest)), ' -s ', num2str(modelLearn), ' -B ', num2str(B)];
        %% train Model
        startTime = tic;
        model = ovrtrain(Y, instance_matrix, cmd);
        stopTime = stopTime + toc(startTime);
        
        %% valid test row
        r = find(YT >= 0);
        
        trueLabel = find(Y >= 0);
        labelVal = unique(Y(trueLabel, :));       
        
        %% test Model
        startTime = tic;
        [pred ac decv] = ovrpredict(YT(r, :), instance_matrixT(r, :), model);
        stopTime = stopTime + toc(startTime);
        
        %% check if this is the best model
        [microCalMatrix microPrecisionVal microRecallVal microF1Val macroPrecisionVal macroRecallVal macroF1Val MCCVal] = microMacroMCCVal(YT(r, :), pred, labelVal(:, 1));             
        if(macroF1Val >= bestMacroF1)
            bestMacroF1 = macroF1Val;
            bestModel = model;
        end
    end
    filename = [num2str(chooseSplit) 'td_one_vs_all_' datasetFolderName '_level_' num2str(nL) '_rewire.mat'];
    fullFileName = fullfile(fn, filename);
    save(fullFileName, '-struct', 'bestModel');
end

sprintf('time taken for model learning: %0.5f mins', stopTime/60)
